@@ -30,6 +30,7 @@ from diffoscope.exc import (
     ContainerExtractionError,
 )
 from diffoscope.tools import tool_required
+from diffoscope.utils import format_cmdline
 from diffoscope.config import Config
 from diffoscope.profiling import profile
 from diffoscope.difference import Difference
@@ -64,7 +65,7 @@ def _run_tests(fold, tests):
     return fold(t(y, x) for x, t, y in tests)
 
 
-class File(object, metaclass=abc.ABCMeta):
+class File(metaclass=abc.ABCMeta):
     if hasattr(magic, 'open'):  # use Magic-file-extensions from file
 
         @classmethod
@@ -172,7 +173,6 @@ class File(object, metaclass=abc.ABCMeta):
 
         return _run_tests(all, all_tests) if all_tests else False
 
-    ENABLE_FALLBACK_RECOGONIZES = True
     FALLBACK_FILE_EXTENSION_SUFFIX = None
     FALLBACK_FILE_TYPE_HEADER_PREFIX = None
 
@@ -195,9 +195,6 @@ class File(object, metaclass=abc.ABCMeta):
             # not valid, they have to re-implement it
             return False
 
-        if not cls.ENABLE_FALLBACK_RECOGONIZES:
-            return False
-
         all_tests = [
             test
             for test in (
@@ -234,9 +231,15 @@ class File(object, metaclass=abc.ABCMeta):
             if hasattr(self, '_other_file'):
                 return self._other_file.__class__.CONTAINER_CLASS(self)
             return None
+
+        def type_name(klass):
+            return "{}.{}".format(klass.__module__, klass.__name__)
+
         if not hasattr(self, '_as_container'):
             logger.debug(
-                'instantiating %s for %s', self.__class__.CONTAINER_CLASS, self
+                'Instantiating a %s for %s',
+                type_name(self.__class__.CONTAINER_CLASS),
+                self.name,
             )
             try:
                 self._as_container = self.__class__.CONTAINER_CLASS(self)
@@ -244,8 +247,8 @@ class File(object, metaclass=abc.ABCMeta):
                 return None
         logger.debug(
             "Returning a %s for %s",
-            self._as_container.__class__.__name__,
-            self,
+            type_name(self._as_container.__class__),
+            self.name,
         )
         return self._as_container
 
@@ -449,17 +452,38 @@ class File(object, metaclass=abc.ABCMeta):
                     )
             except subprocess.CalledProcessError as e:
                 difference = self.compare_bytes(other, source=source)
-                cmd = ' '.join(e.cmd)
                 if difference is None:
                     return None
-                output = '<none>'
-                if e.output:
-                    output = '\n{}'.format(
-                        re.sub(r'^', '    ', e.output, flags=re.MULTILINE)
+
+                # Include either stderr (prefered) or stdout in the hexdump
+                # difference
+                suffix = None
+                for prefix, val in (
+                    ("Standard output", e.stdout),
+                    ("Standard error", e.stderr),
+                ):
+                    if not val:
+                        continue
+                    suffix = ' {}:\n{}'.format(
+                        prefix,
+                        re.sub(
+                            r'^',
+                            '    ',
+                            val.decode('utf-8').strip(),
+                            flags=re.MULTILINE,
+                        ),
                     )
+
+                    # Truncate output
+                    max_len = 250
+                    if len(suffix) > max_len:
+                        suffix = '{}  [...]'.format(suffix[:max_len])
+
                 difference.add_comment(
-                    "Command `{}` exited with return code {}. Output: {}".format(
-                        cmd, e.returncode, output
+                    "Command `{}` exited with exit code {}.{}".format(
+                        format_cmdline(e.cmd),
+                        e.returncode,
+                        suffix or " (No output)",
                     )
                 )
             except RequiredToolNotFound as e:
